@@ -13,6 +13,7 @@ from    functools import partial
 from    time import time
 
 from    p3lib.helper import getHomePath
+from    p3lib.bokeh_auth import SetBokehAuthAttrs
 
 from bokeh.server.server import Server
 from bokeh.application import Application
@@ -22,6 +23,7 @@ from bokeh.models import Range
 from bokeh.palettes import Category20_20 as palette
 from bokeh.resources import Resources
 from bokeh.embed import file_html
+from bokeh.server.auth_provider import AuthModule
 
 from bokeh.plotting import save, output_file
 from bokeh.layouts import gridplot, column, row
@@ -32,13 +34,7 @@ from bokeh.models import Tabs
 from bokeh.models import DataTable, TableColumn
 from bokeh.models import CustomJS
 from bokeh import events
-
-# In bokeh 2.4.8 -> 3.0.3 Panel was removed.
-# TabPanel can be used instead.
-try:
-    from bokeh.models import Panel
-except ImportError:
-    from bokeh.models import TabPanel as Panel
+from bokeh.models import TabPanel
 
 class UpdateEvent(object):
     """@brief Responsible for holding the state of an event sent from a non GUI thread
@@ -96,14 +92,14 @@ class TabbedGUI(object):
                          x_axis_type="datetime",
                          x_axis_location="below",
                          y_range=yrange,
-                         plot_width=width,
-                         plot_height=height)
+                         width=width,
+                         height=height)
         else:
             fig = figure(title=title,
                          x_axis_type="datetime",
                          x_axis_location="below",
-                         plot_width=width,
-                         plot_height=height)
+                         width=width,
+                         height=height)
 
         fig.yaxis.axis_label = yAxisName
         return fig
@@ -249,7 +245,7 @@ class TimeSeriesPlotter(TabbedGUI):
 
         plotPanel = self._getPlotPanel()
 
-        self._tabList.append( Panel(child=plotPanel,  title="Plots") )
+        self._tabList.append( TabPanel(child=plotPanel,  title="Plots") )
         self._doc.add_root( Tabs(tabs=self._tabList) )
         self._doc.add_periodic_callback(self._update, 100)
 
@@ -461,7 +457,7 @@ class SingleAppServer(object):
               the freedom to define your app as required."""
 
     @staticmethod
-    def GetNextUnusedPort(basePort=1024, maxPort = 65534, bindAddress="localhost"):
+    def GetNextUnusedPort(basePort=1024, maxPort = 65534, bindAddress="0.0.0.0"):
         """@brief Get the first unused above the base port.
            @param basePort The port to start checking for available ports.
            @param maxPort The highest port number to check.
@@ -739,3 +735,111 @@ class GUIModel_A(SingleAppServer):
         if time() > self._lastUpdateTime+10:
             self._debug("Updating GUI: Outstanding messages = {}".format( self._plotDataQueue.qsize()) )
             self._lastUpdateTime = time()
+
+
+class MultiAppServer(object):
+    """@brief Responsible for running a bokeh server containing a multiple apps.
+              The server may be started by calling either a blocking or a non
+              blocking method. This provides a basic parent class with
+              the freedom to define your app as required."""
+
+    BOKEH_ALLOW_WS_ORIGIN       = 'BOKEH_ALLOW_WS_ORIGIN'
+
+    @staticmethod
+    def GetNextUnusedPort(basePort=1024, maxPort = 65534, bindAddress="0.0.0.0"):
+        """@brief A helper method to get the first unused above the base port.
+           @param basePort The port to start checking for available ports.
+           @param maxPort The highest port number to check.
+           @param bindAddress The address to bind to.
+           @return The TCP port or -1 if no port is available."""
+        return SingleAppServer.GetNextUnusedPort(basePort=basePort, maxPort=maxPort, bindAddress=bindAddress)
+
+    def __init__(self,
+                 address="0.0.0.0",
+                 bokehPort=0,
+                 wsOrigin="*:*",
+                 credentialsJsonFile=None,
+                 loginHTMLFile="login.html",
+                 accessLogFile=None):
+        """@Constructor
+           @param address The address of the bokeh server.
+           @param bokehPort The TCP port to run the server on. If left at the default
+                  of 0 then a spare TCP port will be used.
+           @param credentialsJsonFile A file that contains the json formatted hashed (via argon2) login credentials.
+           @param accessLogFile The log file to record access to. If left as None then no logging occurs.
+           """
+        if bokehPort == 0:
+            bokehPort = MultiAppServer.GetNextUnusedPort()
+        self._bokehPort=bokehPort
+        self._address=address
+        os.environ[MultiAppServer.BOKEH_ALLOW_WS_ORIGIN]=wsOrigin
+        self._credentialsJsonFile = credentialsJsonFile
+        self._loginHTMLFile = loginHTMLFile
+        self._accessLogFile = accessLogFile
+
+    def getServerPort(self):
+        """@return The bokeh server port."""
+        return self._bokehPort
+
+    def _getAppDict(self, appMethodDict):
+        """@brief Get a dict that can be passed to the Server object to
+                  define the apps to be served."""
+        appDict = {}
+        for key in appMethodDict:
+            appMethod = appMethodDict[key]
+            appDict[key]=Application(FunctionHandler(appMethod))
+        return appDict
+
+    def runBlockingBokehServer(self, appMethodDict, openBrowser=True):
+        """@brief Run the bokeh server. This method will only return when the server shuts down.
+           @param appMethodDict This dict holds references to all the apps yourwish the server
+           to run.
+           The key to each dict entry should be the last part of the URL to point to the app.
+           E.G '/' is the root app which is displayed when the full URL is given.
+           The value should be a reference to the method on this object that holds
+           the app code.
+           @param openBrowser If True then open a browser connected to the / app (default=True)."""
+        appDict = self._getAppDict(appMethodDict)
+
+        #As this gets run in a thread we need to start an event loop
+        evtLoop = asyncio.new_event_loop()
+        asyncio.set_event_loop(evtLoop)
+        if self._credentialsJsonFile:
+            SetBokehAuthAttrs(self._credentialsJsonFile,
+                              self._loginHTMLFile,
+                              accessLogFile=self._accessLogFile)
+            # We don't check the credentials hash file exists as this should have been
+            # done at a higher level. We assume that web server authoristion is required.
+            selfPath = os.path.dirname(os.path.abspath(__file__))
+            authFile = os.path.join(selfPath, "bokeh_auth.py")
+            authModule = AuthModule(authFile)
+            self._server = Server(appDict,
+                                  address=self._address,
+                                  port=self._bokehPort,
+                                  auth_provider=authModule)
+
+        else:
+            self._server = Server(appDict,
+                                  address=self._address,
+                                  port=self._bokehPort)
+
+        self._server.start()
+        if openBrowser:
+            #Show the server in a web browser window
+            self._server.io_loop.add_callback(self._server.show, "/")
+        self._server.io_loop.start()
+
+    def runNonBlockingBokehServer(self, appMethodDict, openBrowser=True):
+        """@brief Run the bokeh server in a separate thread. This is useful
+                  if the we want to load realtime data into the plot from the
+                  main thread.
+           @param @param appMethodDict This dict holds references to all the apps yourwish the server
+           to run.
+           The key to each dict entry should be the last part of the URL to point to the app.
+           E.G '/' is the root app which is displayed when the full URL is given.
+           The value should be a reference to the method on this object that holds
+           the app code.
+           @param openBrowser If True then open a browser connected to the / app (default=True)"""
+        self._serverThread = threading.Thread(target=self.runBlockingBokehServer, args=(appMethodDict,openBrowser))
+        self._serverThread.setDaemon(True)
+        self._serverThread.start()
