@@ -27,52 +27,68 @@ class BootManager(object):
         parser.add_argument(BootManager.CHECK_CMD_OPT,   help="Check the status of an auto started icons_gw instance.", action="store_true", default=False)
 
     @staticmethod
-    def HandleOptions(uio, options, enable_syslog):
+    def HandleOptions(uio, options, enable_syslog, serviceName=None, restartSeconds=1):
         """@brief Handle one of the bot manager command line options if the 
                   user passed it on the cmd line.
            @param uio A UIO instance.
            @param options As returned from parser.parse_args() where parser 
                           is an instance of argparse.ArgumentParser.
            @param enable_syslog True to enable systemd syslog output.
+           @param serviceName The name of the service. If not set then the name of the initially executed 
+                              python file is used.
+           @param restartSeconds The number of seconds to sleep before restarting a service that has stopped (default=1).
            @return True if handled , False if not."""
         handled = False
         if options.check_auto_start:
-            BootManager.CheckAutoStartStatus(uio)
+            BootManager.CheckAutoStartStatus(uio, serviceName)
             handled = True
             
         elif options.enable_auto_start:
-            BootManager.EnableAutoStart(uio, enable_syslog)
+            BootManager.EnableAutoStart(uio, enable_syslog, serviceName, restartSeconds)
             handled = True
             
         elif options.disable_auto_start:
-            BootManager.DisableAutoStart(uio)
+            BootManager.DisableAutoStart(uio, serviceName)
             handled = True
 
         return handled
 
     @staticmethod
-    def EnableAutoStart(uio, enable_syslog):
-        """@brief Enable this program to auto start when the computer on which it is installed starts."""
-        bootManager = BootManager(uio=uio, ensureRootUser=True)
+    def EnableAutoStart(uio, enable_syslog, serviceName, restartSeconds):
+        """@brief Enable this program to auto start when the computer on which it is installed starts.
+           @param uio A UIO instance.
+           @param options As returned from parser.parse_args() where parser 
+                          is an instance of argparse.ArgumentParser.
+           @param enable_syslog True to enable systemd syslog output.
+           @param serviceName The name of the service. If not set then the name of the initially executed 
+                              python file is used.
+           @param restartSeconds The number of seconds to sleep before restarting a service that has stopped (default=1)."""
+        bootManager = BootManager(uio=uio, ensureRootUser=True, serviceName=serviceName, restartSeconds=restartSeconds)
         arsString = " ".join(sys.argv)
         bootManager.add(argString=arsString, enableSyslog=enable_syslog)
 
     @staticmethod
-    def DisableAutoStart(uio):
-        """@brief Enable this program to auto start when the computer on which it is installed starts."""
-        bootManager = BootManager(uio=uio, ensureRootUser=True)
+    def DisableAutoStart(uio, serviceName):
+        """@brief Enable this program to auto start when the computer on which it is installed starts.
+           @param uio A UIO instance.
+           @param serviceName The name of the service. If not set then the name of the initially executed 
+                              python file is used."""
+        bootManager = BootManager(uio=uio, ensureRootUser=True, serviceName=serviceName)
         bootManager.remove()
         
     @staticmethod
-    def CheckAutoStartStatus(uio):
-        """@brief Check the status of a process previously set to auto start."""
-        bootManager = BootManager(uio=uio)
+    def CheckAutoStartStatus(uio, serviceName):
+        """@brief Check the status of a process previously set to auto start.
+           @param uio A UIO instance.
+           @param serviceName The name of the service. If not set then the name of the initially executed 
+                              python file is used."""
+        bootManager = BootManager(uio=uio, serviceName=serviceName)
         lines = bootManager.getStatus()
         if lines and len(lines) > 0:
             for line in lines:
                 uio.info(line)
         
-    def __init__(self, uio=None, allowRootUser=True, ensureRootUser=False):
+    def __init__(self, uio=None, allowRootUser=True, ensureRootUser=False, serviceName=None, restartSeconds=1):
         """@brief Constructor
            @param uio A UIO instance to display user output. If unset then no output
                   is displayed to user.
@@ -81,13 +97,16 @@ class BootManager(object):
                   ensuring programs are started up when a machine boots up 
                   the installed program should be installed for the root 
                   user on Linux systems.
-           @param ensureRootUser If True the current user must be root user (Linux systems)."""
+           @param ensureRootUser If True the current user must be root user (Linux systems).
+           @param serviceName The name of the service. If not set then the name of the initially executed 
+                              python file is used.
+           @param restartSeconds The number of seconds to sleep before restarting a service that has stopped (default=1)."""
         self._uio = uio
         self._allowRootUser=allowRootUser
         self._osName = platform.system()
         self._platformBootManager = None
         if self._osName == BootManager.LINUX_OS_NAME:
-            self._platformBootManager = LinuxBootManager(uio, self._allowRootUser, ensureRootUser)
+            self._platformBootManager = LinuxBootManager(uio, self._allowRootUser, ensureRootUser, serviceName, restartSeconds)
         else:
             raise Exception("{} is an unsupported OS.".format(self._osName) )
 
@@ -155,11 +174,14 @@ class LinuxBootManager(object):
             raise Exception(f"{serviceFolder} folder not found.")
         return serviceFolder
 
-    def __init__(self, uio, allowRootUser, ensureRootUser):
+    def __init__(self, uio, allowRootUser, ensureRootUser, serviceName, restartSeconds):
         """@brief Constructor
            @param uio A UIO instance to display user output. If unset then no output is displayed to user.
            @param allowRootUser If True then allow root user to to auto start programs.
-           @param ensureRootUser If True the current user must be root user."""
+           @param ensureRootUser If True the current user must be root user.
+           @param serviceName The name of the service. If not set then the name of the initially executed 
+                              python file is used.
+           @param restartSeconds The number of seconds to sleep before restarting a service that has stopped."""
         self._uio = uio
         self._logFile = None
         self._allowRootUser=allowRootUser
@@ -180,7 +202,8 @@ class LinuxBootManager(object):
             self._cmdLinePrefix = self._systemCtlBin
         self._username = getpass.getuser()
         self._serviceFolder = LinuxBootManager.GetServiceFolder(self._rootMode)
-        self._appName = None
+        self._serviceName = serviceName
+        self._restartSeconds = restartSeconds
 
     def _getInstallledStartupScript(self):
         """@brief Get the startup script full path. The startup script must be
@@ -267,12 +290,23 @@ class LinuxBootManager(object):
 
         return (appName, absApp)
 
+    def _getServiceName(self):
+        """@brief Get the name of the service.
+           @return The name of the service."""
+        if self._serviceName:
+            serviceName = self._serviceName
+        else:
+            appName, _ = self._getApp()
+            serviceName = appName
+        return "{}.service".format(serviceName)
+
     def _getServiceFile(self, appName):
         """@brief Get the name of the service file.
            @param appName The name of the app to execute.
            @return The absolute path to the service file """
-        serviceName = "{}.service".format(appName)
+        serviceName = self._getServiceName()
         serviceFile = os.path.join(self._serviceFolder, serviceName)
+        self._uio.info(f"SERVICE FILE: {serviceFile}")
         return serviceFile
 
     def add(self, user, argString=None, enableSyslog=False):
@@ -291,8 +325,9 @@ class LinuxBootManager(object):
             user = self._username
 
         appName, absApp = self._getApp()
+        serviceName = self._getServiceName()
 
-        serviceFile = self._getServiceFile(appName)
+        serviceFile = self._getServiceFile(serviceName)
 
         lines = []
         lines.append("[Unit]")
@@ -302,7 +337,7 @@ class LinuxBootManager(object):
         lines.append("[Service]")
         lines.append("Type=simple")
         lines.append("Restart=always")
-        lines.append("RestartSec=1")
+        lines.append(f"RestartSec={self._restartSeconds}")
         if enableSyslog:
             lines.append("StandardOutput=syslog")
             lines.append("StandardError=syslog")
@@ -342,12 +377,12 @@ class LinuxBootManager(object):
 
         cmd = "{} daemon-reload".format(self._cmdLinePrefix)
         self._runLocalCmd(cmd)
-        cmd = "{} enable {}".format(self._cmdLinePrefix, appName)
-        self._info("Enabled {} on restart".format(appName))
+        cmd = "{} enable {}".format(self._cmdLinePrefix, serviceName)
+        self._info("Enabled {} on restart".format(serviceName))
         self._runLocalCmd(cmd)
-        cmd = "{} start {}".format(self._cmdLinePrefix, appName)
+        cmd = "{} start {}".format(self._cmdLinePrefix, serviceName)
         self._runLocalCmd(cmd)
-        self._info("Started {}".format(appName))
+        self._info("Started {}".format(serviceName))
 
     def remove(self):
         """@brief Remove the executable file to the processes started at boot time.
@@ -355,29 +390,30 @@ class LinuxBootManager(object):
                   named the same as the python file executed without the .py suffix."""
         appName, _ = self._getApp()
 
+        serviceName = self._getServiceName()
         serviceFile = self._getServiceFile(appName)
         if os.path.isfile(serviceFile):
-            cmd = "{} disable {}".format(self._cmdLinePrefix, appName)
+            cmd = "{} disable {}".format(self._cmdLinePrefix, serviceName)
             self._runLocalCmd(cmd)
-            self._info("Disabled {} on restart".format(appName))
+            self._info("Disabled {} on restart".format(serviceName))
 
-            cmd = "{} stop {}".format(self._cmdLinePrefix, appName)
+            cmd = "{} stop {}".format(self._cmdLinePrefix, serviceName)
             self._runLocalCmd(cmd)
-            self._info("Stopped {}".format(appName))
+            self._info("Stopped {}".format(serviceName))
 
             os.remove(serviceFile)
             self._log("Removed {}".format(serviceFile))
         else:
-            self._info("{} service not found".format(appName))
+            self._info("{} service not found".format(serviceName))
 
     def getStatusLines(self):
         """@brief Get a status report.
            @return Lines of text indicating the status of a previously started process."""
-        appName, _ = self._getApp()
+        serviceName = self._getServiceName()
         if self._rootMode:
-            p = Popen([self._systemCtlBin, 'status', appName], stdin=PIPE, stdout=PIPE, stderr=PIPE)
+            p = Popen([self._systemCtlBin, 'status', serviceName], stdin=PIPE, stdout=PIPE, stderr=PIPE)
         else:
-            p = Popen([self._systemCtlBin, '--user', 'status', appName], stdin=PIPE, stdout=PIPE, stderr=PIPE)
+            p = Popen([self._systemCtlBin, '--user', 'status', serviceName], stdin=PIPE, stdout=PIPE, stderr=PIPE)
         output, err = p.communicate(b"input data that is passed to subprocess' stdin")
         response = output.decode() + "\n" + err.decode()
         lines = response.split("\n")
