@@ -1,7 +1,7 @@
 # !/usr/bin/env python3
 
 """NiceGui Tools
-   Responsible for providing helper classes for nicegui interfaces 
+   Responsible for providing helper classes for nicegui interfaces
    aimed at reducing coding required for a GUI.
 """
 
@@ -12,6 +12,9 @@ import platform
 from time import sleep
 from queue import Queue
 from time import time, strftime, localtime
+from pathlib import Path
+
+from p3lib.helper import getProgramVersion
 
 from nicegui import ui
 
@@ -19,16 +22,17 @@ class TabbedNiceGui(object):
     """@brief Responsible for starting the providing a tabbed GUI.
               This class is designed to ease the creation of a tabbed GUI interface.
               The contents of each tab can be defined in the subclass.
-              The GUI includes a message log area below each tab. Tasks can send messages 
+              The GUI includes a message log area below each tab. Tasks can send messages
               to this log area.
-              If a subclass sets the self._logFile attributed then all messages sent to the 
+              If a subclass sets the self._logFile attributed then all messages sent to the
               log area are written to a log file with timestamps."""
 
     # This can be used in the markdown text for a TAB description to give slightly larger text
     # than normal.
     DESCRIP_STYLE               = '<span style="font-size:1.2em;">'
     ENABLE_BUTTONS              = "ENABLE_BUTTONS"
-    NOTIFY_DIALOG               = "NOTIFY_DIALOG"
+    NOTIFY_DIALOG_INFO          = "NOTIFY_DIALOG_INFO"
+    NOTIFY_DIALOG_ERROR         = "NOTIFY_DIALOG_ERROR"
     UPDATE_SECONDS              = "UPDATE_SECONDS"
     INFO_MESSAGE                = "INFO:  "
     WARN_MESSAGE                = "WARN:  "
@@ -47,7 +51,7 @@ class TabbedNiceGui(object):
     def GetDateTimeStamp():
         """@return The log file date/time stamp """
         return strftime("%Y%m%d%H%M%S", localtime()).lower()
-    
+
     @staticmethod
     def GetInstallFolder():
         """@return The folder where the apps are installed."""
@@ -55,7 +59,7 @@ class TabbedNiceGui(object):
         if not os.path.isdir(installFolder):
             raise Exception(f"{installFolder} folder not found.")
         return installFolder
-    
+
     @staticmethod
     def GetLogFileName(logFilePrefix):
         """@param logFilePrefix The text in the log file name before the timestamp.
@@ -63,7 +67,7 @@ class TabbedNiceGui(object):
         dateTimeStamp = TabbedNiceGui.GetDateTimeStamp()
         logFileName = f"{logFilePrefix}_{dateTimeStamp}.log"
         return logFileName
-    
+
     @staticmethod
     def CheckPort(port):
         """@brief Check the server port.
@@ -72,34 +76,12 @@ class TabbedNiceGui(object):
             raise Exception("The minimum TCP port that you can bind the GUI server to is 1024.")
         if port > 65535:
             raise Exception("The maximum TCP port that you can bind the GUI server to is 65535.")
-        
+
     @staticmethod
     def GetProgramVersion():
         """@brief Get the program version from the poetry pyproject.toml file.
            @return The version of the installed program (string value)."""
-        poetryConfigFile = os.path.join(TabbedNiceGui.LOCAL_PATH, TabbedNiceGui.POETRY_CONFIG_FILE)
-        if not os.path.isfile(poetryConfigFile):
-            poetryConfigFile = os.path.join(TabbedNiceGui.LOCAL_PATH, ".." + os.sep + TabbedNiceGui.POETRY_CONFIG_FILE)
-            poetryConfigFile2 = poetryConfigFile
-            if not os.path.isfile(poetryConfigFile):
-                cwd = os.getcwd()
-                poetryConfigFile = os.path.join(cwd, TabbedNiceGui.POETRY_CONFIG_FILE)
-                if not os.path.isfile(poetryConfigFile):
-                    raise Exception(f"{poetryConfigFile}, {poetryConfigFile2} and {poetryConfigFile} not found.")
-                
-        programVersion = None
-        with open(poetryConfigFile, 'r') as fd:
-            lines = fd.readlines()
-            for line in lines:
-                line=line.strip("\r\n")
-                if line.startswith('version'):
-                    elems = line.split("=")
-                    if len(elems) == 2:
-                        programVersion = elems[1].strip('" ')
-                        break
-        if programVersion is None:
-            raise Exception(f"Failed to extract program version from '{line}' line of {poetryConfigFile} file.")
-        return programVersion
+        return getProgramVersion()
 
     def __init__(self, debugEnabled, logPath=None):
         """@brief Constructor
@@ -144,8 +126,8 @@ class TabbedNiceGui(object):
 
     # Start ------------------------------
     # Methods that allow the GUI to display standard UIO messages
-    # This allows the GUI to be used with code that was written 
-    # to be used on the command line using UIO class instances 
+    # This allows the GUI to be used with code that was written
+    # to be used on the command line using UIO class instances
     #
     def info(self, msg):
         """@brief Send a info message to be displayed in the GUI.
@@ -160,19 +142,31 @@ class TabbedNiceGui(object):
            @param msg The message to be displayed."""
         msgDict = {TabbedNiceGui.WARN_MESSAGE: str(msg)}
         self.updateGUI(msgDict)
-        
+
     def error(self, msg):
         """@brief Send a error message to be displayed in the GUI.
                   This can be called from outside the GUI thread.
            @param msg The message to be displayed."""
         msgDict = {TabbedNiceGui.ERROR_MESSAGE: str(msg)}
         self.updateGUI(msgDict)
-        
-    def infoDialog(self, msg):
+
+    def infoDialog(self, msg, addToMessageLog=True):
         """@brief Display an info level dialog.
-           @param msg The message dialog."""
-        msgDict = {TabbedNiceGui.NOTIFY_DIALOG: str(msg)}
+           @param msg The message dialog.
+           @param addToMessageLog If True the message will also be added to the message log."""
+        msgDict = {TabbedNiceGui.NOTIFY_DIALOG_INFO: str(msg)}
         self.updateGUI(msgDict)
+        if addToMessageLog:
+            self.info(msg)
+
+    def errorDialog(self, msg, addToMessageLog=True):
+        """@brief Display an error level dialog.
+           @param msg The message dialog.
+           @param addToMessageLog If True the message will also be added to the message log."""
+        msgDict = {TabbedNiceGui.NOTIFY_DIALOG_ERROR: str(msg)}
+        self.updateGUI(msgDict)
+        if addToMessageLog:
+            self.error(msg)
 
     def debug(self, msg):
         """@brief Send a debug message to be displayed in the GUI.
@@ -198,7 +192,7 @@ class TabbedNiceGui(object):
         else:
             returnText = inputObj.value
         return returnText
-            
+
     def reportException(self, exception):
         """@brief Report an exception.
                   If debug is enabled a full stack trace is displayed.
@@ -235,18 +229,18 @@ class TabbedNiceGui(object):
         # Check we have a table to display
         if len(table) == 0:
             raise Exception("No table rows to display")
-        
+
         # Check all rows have the same number of columns in the table
         colCount = len(table[0])
         for row in table:
             if len(row) != colCount:
                 raise Exception(f"{str(row)} column count different from first row ({colCount})")
-        
+
         for row in table:
             for col in row:
                 if not isinstance(col, str):
                     raise Exception(f"Table column is not a string: {col} in {row}")
-                
+
         # Get the max width for each column
         for col in range(0,colCount):
             maxWidth=0
@@ -258,10 +252,10 @@ class TabbedNiceGui(object):
         tableWidth = 1
         for columnWidth in columnWidths:
             tableWidth += columnWidth + 3 # Space each side of the column + a column divider character
-                    
+
         # Add the top line of the table
         self.info(rowSeparatorChar*tableWidth)
-               
+
         # The starting row index
         for rowIndex in range(0, len(table)):
             rowText = colSeparatorChar
@@ -280,7 +274,10 @@ class TabbedNiceGui(object):
     def setLogFile(self, logFile):
         pass
 
-    # End ------------------------------   
+    def storeToDebugLog(self, msg):
+        pass
+
+    # End ------------------------------
 
     def _saveLogMsg(self, msg):
         """@brief Save the message to a log file.
@@ -295,7 +292,7 @@ class TabbedNiceGui(object):
             with open(self._logFile, 'a') as fd:
                 dateTimeStamp = TabbedNiceGui.GetDateTimeStamp()
                 fd.write(dateTimeStamp + ": " + msg + '\n')
-    
+
     def _getDisplayMsg(self, msg, prefix):
         """@brief Get the msg to display. If the msg does not already have a msg level we add one.
            @param msg The source msg.
@@ -315,8 +312,16 @@ class TabbedNiceGui(object):
         self._log.push(msg)
         self._saveLogMsg(msg)
         self._logMessageCount += 1
-        # We've received a log message so update progress bar if required. 
+        # We've received a log message so update progress bar if required.
         self._updateProgressBar(msg)
+        # Wait a moment for DOM to update, then scroll to the end of the log
+        # so that the message just added is visible.
+        ui.timer(0.05, lambda: ui.run_javascript("""
+            const el = document.querySelector('.my-log');
+            if (el) {
+                el.scrollTop = el.scrollHeight;
+            }
+        """), once=True)
 
     def _infoGT(self, msg):
         """@brief Update an info level message. This must be called from the GUI thread.
@@ -374,58 +379,70 @@ class TabbedNiceGui(object):
             if isinstance(rxMessage, dict):
                 self._processRXDict(rxMessage)
 
-    def initGUI(self, tabNameList, tabMethodInitList, reload=True, address="0.0.0.0", port=DEFAULT_SERVER_PORT, pageTitle="NiceGUI"):
+    def initGUI(self,
+                tabNameList,
+                tabMethodInitList,
+                reload=True,
+                address="0.0.0.0",
+                port=DEFAULT_SERVER_PORT,
+                pageTitle="NiceGUI"):
         """@brief Init the tabbed GUI.
            @param tabNameList A list of the names of each tab to be created.
-           @param tabMethodInitList A list of the methods to be called to init each of the above tabs. 
+           @param tabMethodInitList A list of the methods to be called to init each of the above tabs.
                                     The two lists must be the same size.
            @param reload If reload is set False then changes to python files will not cause the server to be restarted.
            @param address The address to bind the server to.
            @param The TCP port to bind the server to.
-           @param pageTitle The page title that appears in the browser."""
-        # A bit of defensive programming.
-        if len(tabNameList) != len(tabMethodInitList):
-            raise Exception(f"initGUI: BUG: tabNameList ({len(tabNameList)}) and tabMethodInitList ({len(tabMethodInitList)}) are not the same length.")
-        tabObjList = []
-        with ui.row():
-            with ui.tabs().classes('w-full') as tabs:
-                for tabName in tabNameList:
-                    tabObj = ui.tab(tabName)
-                    tabObjList.append(tabObj)
-                
-            with ui.tab_panels(tabs, value=tabObjList[0]).classes('w-full'):
-                for tabObj in tabObjList:
-                    with ui.tab_panel(tabObj):
-                        tabIndex = tabObjList.index(tabObj)
-                        tabMethodInitList[tabIndex]()
+           @param pageTitle The page title that appears in the browser.
+           @param maxLogLines The maximum number of lines to be displayed in the log. Be aware setting this higher will cause the browser to use more memory."""
+        with ui.column().classes('h-screen w-screen p4'):
+            # A bit of defensive programming.
+            if len(tabNameList) != len(tabMethodInitList):
+                raise Exception(f"initGUI: BUG: tabNameList ({len(tabNameList)}) and tabMethodInitList ({len(tabMethodInitList)}) are not the same length.")
+            tabObjList = []
+            with ui.row():
+                with ui.tabs().classes('w-full') as tabs:
+                    for tabName in tabNameList:
+                        tabObj = ui.tab(tabName)
+                        tabObjList.append(tabObj)
 
-        guiLogLevel = "warning"
-        if self._debugEnabled:
-            guiLogLevel = "debug"
+                with ui.tab_panels(tabs, value=tabObjList[0]).classes('w-full'):
+                    for tabObj in tabObjList:
+                        with ui.tab_panel(tabObj):
+                            tabIndex = tabObjList.index(tabObj)
+                            tabMethodInitList[tabIndex]()
 
-        ui.label("Message Log")
-        self._progress = ui.slider(min=0,max=TabbedNiceGui.MAX_PROGRESS_VALUE,step=1)
-        self._progress.set_visibility(False)
-        self._progress.min = 0
-        # Don't allow user to adjust progress bar thumb
-        self._progress.disable()
-        self._log = ui.log(max_lines=2000)
-        self._log.set_visibility(True)
+            guiLogLevel = "warning"
+            if self._debugEnabled:
+                guiLogLevel = "debug"
 
-        with ui.row():
-            ui.button('Clear Log', on_click=self._clearLog)
-            ui.button('Log Message Count', on_click=self._showLogMsgCount)
-            ui.button('Quit', on_click=self.close)
+            ui.label("Message Log")
+            self._progress = ui.slider(min=0,max=TabbedNiceGui.MAX_PROGRESS_VALUE,step=1)
+            self._progress.set_visibility(False)
+            self._progress.min = 0
+            # Don't allow user to adjust progress bar thumb
+            self._progress.disable()
+            # Setup the log area to fill the available space in the page vertically and horizontally
+            # The 32px is to make space for the vertical scrollbar within the page or it will be
+            # shifted out of sight to the right.
+            # Previously used self._log = ui.log(max_lines=2000) but the ui.log() does not currently limit data in the log.
+            self._log = ui.log().classes('my-log grow w-full max-w-[calc(100%-32px)] overflow-auto box-border')
+            self._log.set_visibility(True)
 
-        with ui.row():
-            ui.label(f"Software Version: {self._programVersion}")
+            with ui.row():
+                ui.button('Clear Log', on_click=self._clearLog)
+                ui.button('Log Message Count', on_click=self._showLogMsgCount)
+                ui.button('Quit', on_click=self.close)
 
-        ui.timer(interval=TabbedNiceGui.GUI_TIMER_SECONDS, callback=self.guiTimerCallback)
-        ui.timer(interval=TabbedNiceGui.PROGRESS_TIMER_SECONDS, callback=self.progressTimerCallback)
-        ui.run(host=address, port=port, title=pageTitle, dark=True, uvicorn_logging_level=guiLogLevel, reload=reload)
+            with ui.row():
+                ui.label(f"Software Version: {self._programVersion}")
+
+            ui.timer(interval=TabbedNiceGui.GUI_TIMER_SECONDS, callback=self.guiTimerCallback)
+            ui.timer(interval=TabbedNiceGui.PROGRESS_TIMER_SECONDS, callback=self.progressTimerCallback)
+            ui.run(host=address, port=port, title=pageTitle, dark=True, uvicorn_logging_level=guiLogLevel, reload=reload)
 
     def progressTimerCallback(self):
-        """@brief Time to update the progress bar. We run the timer all the time because there appears to be a 
+        """@brief Time to update the progress bar. We run the timer all the time because there appears to be a
                   bug in the ui.timer instance. Calling cancel() does not appear to cancel the timer."""
         if self._updateProgressOnTimer and self._progress.visible:
             # Increment the progress bar
@@ -433,17 +450,17 @@ class TabbedNiceGui(object):
 
     def _startProgress(self, durationSeconds=0, startMessage=None, expectedMsgList=[], expectedMsgCount=0):
         """@brief Start a timer that will update the progress bar.
-                  The progress bar can simply update on a timer every second with durationSeconds set to the expected length 
+                  The progress bar can simply update on a timer every second with durationSeconds set to the expected length
                   of the task.
 
-                  If startMessage is set to a text string the progress time will not start until the log message area contains 
+                  If startMessage is set to a text string the progress time will not start until the log message area contains
                   the start message.
-                  
-                  Alternatively if expectedMsgList contains a list of strings we expect to receive then the progress bar is 
+
+                  Alternatively if expectedMsgList contains a list of strings we expect to receive then the progress bar is
                   updated as each message is received. The messages may be the entire line of a log message or parts of a
                   log message line.
 
-                  Alternatively if expectedMsgCount is set to a value > 0 then the progress bar is updated as each message is 
+                  Alternatively if expectedMsgCount is set to a value > 0 then the progress bar is updated as each message is
                   added to the log and reaches 100% when the number of messages added to the log file reaches the expectedMsgCount.
 
             @param startMessage The text of the log message we expect to receive to trigger the progress bar timer start.
@@ -495,7 +512,7 @@ class TabbedNiceGui(object):
         if self._expectedProgressBarMsgCount > 0:
             self._progressValue = self._progressValue + self._progressStepValue
             self._progress.set_value( self._progressValue )
-            
+
         # If we have a list of log messages to update the progress bar.
         elif len(self._progressBarExpectedMessageList) > 0:
             if self._expectedProgressBarMessageIndex < len(self._progressBarExpectedMessageList):
@@ -517,7 +534,7 @@ class TabbedNiceGui(object):
         """@brief Should be called before a task is started."""
         self._enableAllButtons(False)
         self._clearMessages()
-        
+
     def _clearLog(self):
         """@brief Clear the log text"""
         if self._log:
@@ -526,11 +543,11 @@ class TabbedNiceGui(object):
     def _showLogMsgCount(self):
         """@brief Show the number of log messages"""
         ui.notify(f"{self._getLogMessageCount()} messages in the log.")
-        
+
     def close(self):
         """@brief Close down the app server."""
         ui.notify("Press 'CTRL C' at command line or close the terminal window to quit.")
-        # A subclass close() method can call 
+        # A subclass close() method can call
         # app.shutdown()
         # if reload=False on ui.run()
 
@@ -557,15 +574,19 @@ class TabbedNiceGui(object):
         elif TabbedNiceGui.DEBUG_MESSAGE in rxDict:
             msg = rxDict[TabbedNiceGui.DEBUG_MESSAGE]
             self._debugGT(msg)
-            
+
         elif TabbedNiceGui.ENABLE_BUTTONS in rxDict:
             state = rxDict[TabbedNiceGui.ENABLE_BUTTONS]
             self._enableAllButtons(state)
 
-        elif TabbedNiceGui.NOTIFY_DIALOG in rxDict:
-            message = rxDict[TabbedNiceGui.NOTIFY_DIALOG]
+        elif TabbedNiceGui.NOTIFY_DIALOG_INFO in rxDict:
+            message = rxDict[TabbedNiceGui.NOTIFY_DIALOG_INFO]
             ui.notify(message, close_button='OK', type="positive", position="center")
- 
+
+        elif TabbedNiceGui.NOTIFY_DIALOG_ERROR in rxDict:
+            message = rxDict[TabbedNiceGui.NOTIFY_DIALOG_ERROR]
+            ui.notify(message, close_button='OK', type="negative", position="center")
+
         else:
 
             self._handleGUIUpdate(rxDict)
@@ -601,47 +622,50 @@ class TabbedNiceGui(object):
 
             elif time() >= timeoutT:
                 raise Exception(f"{timeout} second GUI response timeout.")
-            
+
             else:
                 # Don't spin to fast
                 sleep(0.1)
-            
+
         return rxDict
-    
+
     def _handleGUIUpdate(self, rxDict):
         """@brief Process the dicts received from the GUI message queue
                   that were not handled by the parent class.
            @param rxDict The dict received from the GUI message queue."""
         raise NotImplementedError("_handleGUIUpdate() is not implemented. Implement this method in a subclass of TabbedNiceGUI")
-    
+
 
 class YesNoDialog(object):
     """@brief Responsible for displaying a dialog box to the user with a boolean (I.E yes/no, ok/cancel) response."""
-    TEXT_INPUT_FIELD_TYPE   = 1
-    NUMBER_INPUT_FIELD_TYPE = 2
-    SWITCH_INPUT_FIELD_TYPE = 3
-    DROPDOWN_INPUT_FIELD    = 4
-    COLOR_INPUT_FIELD       = 5
-    DATE_INPUT_FIELD        = 6
-    TIME_INPUT_FIELD        = 7
-    KNOB_INPUT_FIELD        = 8
-    VALID_FIELD_TYPE_LIST   = (TEXT_INPUT_FIELD_TYPE, 
-                               NUMBER_INPUT_FIELD_TYPE, 
-                               SWITCH_INPUT_FIELD_TYPE, 
+    TEXT_INPUT_FIELD_TYPE       = 1
+    NUMBER_INPUT_FIELD_TYPE     = 2
+    SWITCH_INPUT_FIELD_TYPE     = 3
+    DROPDOWN_INPUT_FIELD        = 4
+    COLOR_INPUT_FIELD           = 5
+    DATE_INPUT_FIELD            = 6
+    TIME_INPUT_FIELD            = 7
+    KNOB_INPUT_FIELD            = 8
+    HOUR_MIN_INPUT_FIELD_TYPE   = 9
+    VALID_FIELD_TYPE_LIST   = (TEXT_INPUT_FIELD_TYPE,
+                               NUMBER_INPUT_FIELD_TYPE,
+                               SWITCH_INPUT_FIELD_TYPE,
                                DROPDOWN_INPUT_FIELD,
                                COLOR_INPUT_FIELD,
                                DATE_INPUT_FIELD,
                                TIME_INPUT_FIELD,
-                               KNOB_INPUT_FIELD)
+                               KNOB_INPUT_FIELD,
+                               HOUR_MIN_INPUT_FIELD_TYPE)
 
     FIELD_TYPE_KEY          = "FIELD_TYPE_KEY"      # The type of field to be displayed.
-    VALUE_KEY               = "VALUE_KEY"           # The value to be displayed in the field when the dialog is displayed. 
+    VALUE_KEY               = "VALUE_KEY"           # The value to be displayed in the field when the dialog is displayed.
     MIN_NUMBER_KEY          = "MIN_NUMBER_KEY"      # If the type is NUMBER_INPUT_FIELD_TYPE, the min value that can be entered.
     MAX_NUMBER_KEY          = "MAX_NUMBER_KEY"      # If the type is NUMBER_INPUT_FIELD_TYPE, the max value that can be entered.
     WIDGET_KEY              = "WIDGET_KEY"          # The key to the GUI widget (E.G ui.input, ui.number etc)
     OPTIONS_KEY             = "OPTIONS_KEY"         # Some input fields require a list of options (E.G DROPDOWN_INPUT_FIELD).
+    STEP_KEY                = "STEP_KEY"            # The step size for numerical input fields
 
-    def __init__(self, 
+    def __init__(self,
                  prompt,
                  successMethod,
                  failureMethod=None,
@@ -656,7 +680,7 @@ class YesNoDialog(object):
         self._successMethod          = None          # The method to be called when the success button is selected.
         self._failureMethod          = None          # The method to be called when the failure button is selected.
         self._inputFieldDict         = {}            # A dict of input field details to be included in the dialog. Can be left as an empty dict if no input fields are required.
-                                                     # The key in this dict is the name of the input field that the user sees. 
+                                                     # The key in this dict is the name of the input field that the user sees.
                                                      # The value in this dict is another dict containing details of the input field which may be
 
         self.setPrompt(prompt)
@@ -666,13 +690,14 @@ class YesNoDialog(object):
         self.setFailureButtonLabel(failureButtonText)
 
 
-    def addField(self, name, fieldType, value=None, minNumber=None, maxNumber=None, options=None):
+    def addField(self, name, fieldType, value=None, minNumber=None, maxNumber=None, options=None, step=1):
         """@brief Add a field to the dialog.
            @param name          The name of the field to be added.
            @param fieldType     The type of field to be entered.
            @param value         The optional initial value for the field when the dialog is displayed.
            @param minNumber     The optional min value if the fieldType = NUMBER_INPUT_FIELD_TYPE.
            @param maxNumber     The optional max value if the fieldType = NUMBER_INPUT_FIELD_TYPE.
+           @param step          The step size for numerical input fields.
            """
         if name and len(name) > 0:
             if fieldType in YesNoDialog.VALID_FIELD_TYPE_LIST:
@@ -680,14 +705,15 @@ class YesNoDialog(object):
                                               YesNoDialog.VALUE_KEY:          value,
                                               YesNoDialog.MIN_NUMBER_KEY:     minNumber,
                                               YesNoDialog.MAX_NUMBER_KEY:     maxNumber,
-                                              YesNoDialog.OPTIONS_KEY:        options}
+                                              YesNoDialog.OPTIONS_KEY:        options,
+                                              YesNoDialog.STEP_KEY:           step}
 
             else:
                 raise Exception(f"YesNoDialog.addField() {fieldType} is an invalid field type.")
 
         else:
             raise Exception("YesNoDialog.addField() name not set.")
-        
+
     def _init(self):
         """@brief Init the dialog."""
         with ui.dialog() as self._dialog, ui.card():
@@ -701,11 +727,13 @@ class YesNoDialog(object):
                     value = self._inputFieldDict[fieldName][YesNoDialog.VALUE_KEY]
                     min = self._inputFieldDict[fieldName][YesNoDialog.MIN_NUMBER_KEY]
                     max = self._inputFieldDict[fieldName][YesNoDialog.MAX_NUMBER_KEY]
-                    widget = ui.number(label=fieldName, 
-                                        value=value, 
-                                        min=min, 
-                                        max=max).style('width: 200px;')
-                    
+                    step = self._inputFieldDict[fieldName][YesNoDialog.STEP_KEY]
+                    widget = ui.number(label=fieldName,
+                                        value=value,
+                                        min=min,
+                                        max=max,
+                                        step=step).style('width: 200px;')
+
                 elif fieldType == YesNoDialog.SWITCH_INPUT_FIELD_TYPE:
                     widget = ui.switch(fieldName)
 
@@ -717,7 +745,7 @@ class YesNoDialog(object):
                         widget.tooltip(fieldName)
                     else:
                         raise Exception("BUG: DROPDOWN_INPUT_FIELD defined without defining the options.")
-                    
+
                 elif fieldType == YesNoDialog.COLOR_INPUT_FIELD:
                     widget = ui.color_input(label=fieldName)
 
@@ -733,6 +761,10 @@ class YesNoDialog(object):
                     widget = ui.knob(show_value=True)
                     widget.tooltip(fieldName)
 
+                elif fieldType == YesNoDialog.HOUR_MIN_INPUT_FIELD_TYPE:
+                    widget = self._get_input_time_field(fieldName)
+                    widget.tooltip(fieldName)
+
                 # Save a ref to the widet in the field dict
                 self._inputFieldDict[fieldName][YesNoDialog.WIDGET_KEY] = widget
 
@@ -745,6 +777,25 @@ class YesNoDialog(object):
                 ui.button(self._successButtonText, on_click=self._internalSuccessMethod)
                 ui.button(self._failureButtonText,  on_click=self._internalFailureMethod)
 
+    def _get_input_time_field(self, label):
+        """@brief Add a control to allow the user to enter the time as an hour and min.
+           @param label The label for the time field.
+           @return The input field containing the hour and minute entered."""
+        # Put this off the bottom of the mobile screen as most times it will not be needed
+        # and there is not enough room on the mobile screen above the plot pane.
+        with ui.row().classes('w-full'):
+            ui.label(label)
+            with ui.row().classes('w-full'):
+                time_input = ui.input("Time (HH:MM)")
+                with time_input as time:
+                    with ui.menu().props('no-parent-event') as menu:
+                        with ui.time().bind_value(time):
+                            with ui.row().classes('justify-end'):
+                                ui.button('Close', on_click=menu.close).props('flat')
+                    with time.add_slot('append'):
+                        ui.icon('access_time').on('click', menu.open).classes('cursor-pointer')
+        return time_input
+
     def setPrompt(self, prompt):
         """@brief Set the user prompt.
            @param prompt The user prompt."""
@@ -769,7 +820,7 @@ class YesNoDialog(object):
         """@brief Set the text of the failure button.
            @param label The failure button text."""
         self._failureButtonText = label
-        
+
     def show(self):
         """@brief Allow the user to select yes/no, ok/cancel etc in response to a question."""
         self._init()
@@ -796,7 +847,7 @@ class YesNoDialog(object):
             widget = self._inputFieldDict[fieldName][YesNoDialog.WIDGET_KEY]
             if hasattr(widget, 'value'):
                 self._inputFieldDict[fieldName][YesNoDialog.VALUE_KEY] = self._inputFieldDict[fieldName][YesNoDialog.WIDGET_KEY].value
-        # If defined call the method 
+        # If defined call the method
         if self._successMethod:
             self._successMethod()
 
@@ -809,3 +860,110 @@ class YesNoDialog(object):
     def close(self):
         """@brief Close the boolean dialog."""
         self._dialog.close()
+
+class local_file_picker(ui.dialog):
+    """@brief Allows the user to select local files and folders.
+       This is is a change to https://github.com/zauberzeug/nicegui/blob/main/examples/local_file_picker/local_file_picker.py"""
+
+    def __init__(self,
+                 directory: str,
+                 *,
+                 upper_limit: str = None,
+                 multiple: bool = False,
+                 show_hidden_files: bool = False) -> None:
+        """Local File Picker
+
+        This is a simple file picker that allows you to select a file from the local filesystem where NiceGUI is running.
+
+        :param directory: The directory to start in.
+        :param upper_limit: The directory to stop at (None: no limit, default: same as the starting directory).
+        :param multiple: Whether to allow multiple files to be selected.
+        :param show_hidden_files: Whether to show hidden files.
+        """
+        super().__init__()
+        self._selected_drive = None
+        # If on a Windows platform attempt to separate the drive and the directory
+        if platform.system() == 'Windows' and directory and len(directory) >= 2:
+            if directory[0].isalpha() and directory[1] == ':':
+                self._selected_drive = directory[:2]
+                directory = directory[2:]
+
+        self.path = Path(directory).expanduser()
+        if upper_limit is None:
+            self.upper_limit = None
+        else:
+            self.upper_limit = Path(directory if upper_limit == ... else upper_limit).expanduser()
+        self.show_hidden_files = show_hidden_files
+
+        with self, ui.card().style('overflow-x: auto; max-width: 100%;'):
+            self.add_drives_toggle()
+            self.grid = ui.aggrid({
+                'columnDefs': [{'field': 'name', 'headerName': 'File'}],
+                'rowSelection': 'multiple' if multiple else 'single',
+            }, html_columns=[0]).style('min-width: 600px')
+            self.grid.on('cellDoubleClicked', self.handle_double_click)
+            with ui.row().classes('w-full justify-end'):
+                self._select_folder_checkbox = ui.switch('Select Folder')
+                #self._select_folder_checkbox = ui.checkbox('Folder')
+                self._select_folder_checkbox.tooltip("Select this if you wish to select a folder.")
+                ui.button('Cancel', on_click=self.close).props('outline')
+
+        self.update_grid()
+
+    def add_drives_toggle(self):
+        if platform.system() == 'Windows':
+            import win32api
+            drives = win32api.GetLogicalDriveStrings().split('\000')[:-1]
+            drive = drives[0]
+            # If the caller passed a drive letter select this drive
+            if self._selected_drive:
+                for drive in drives:
+                    if drive.startswith(self._selected_drive):
+                        break
+            self.drives_toggle = ui.toggle(drives, value=drive, on_change=self.update_drive)
+
+        # Display the current path
+        self._path_label = ui.label(str(self.path))
+
+    def update_drive(self):
+        self.path = Path(self.drives_toggle.value).expanduser()
+        self.update_grid()
+
+    def update_grid(self) -> None:
+        paths = list(self.path.glob('*'))
+        if not self.show_hidden_files:
+            paths = [p for p in paths if not p.name.startswith('.')]
+        paths.sort(key=lambda p: p.name.lower())
+        paths.sort(key=lambda p: not p.is_dir())
+
+        self.grid.options['rowData'] = [
+            {
+                'name': f'📁 <strong>{p.name}</strong>' if p.is_dir() else p.name,
+                'path': str(p),
+            }
+            for p in paths
+        ]
+        if (self.upper_limit is None and self.path != self.path.parent) or \
+                (self.upper_limit is not None and self.path != self.upper_limit):
+            self.grid.options['rowData'].insert(0, {
+                'name': '📁 <strong>..</strong>',
+                'path': str(self.path.parent),
+            })
+        self.grid.update()
+
+    def handle_double_click(self, e) -> None:
+        self.path = Path(e.args['data']['path'])
+        if self.path.is_dir() and not self._select_folder_checkbox.value:
+            self.update_grid()
+        else:
+            selected = str(self.path)
+            # If Windows platform add the drive letter to the path
+            if platform.system() == 'Windows':
+                if selected.startswith('\\'):
+                    selected=selected[1:]
+                    selected = self.drives_toggle.value + selected
+
+            self.submit([selected])
+
+        # Update the displayed path
+        self._path_label.set_text(str(self.path))
\ No newline at end of file
