@@ -4,6 +4,7 @@ import sys
 import os
 import re
 import traceback
+import platform
 from   threading import Lock
 from   socket import socket, AF_INET, SOCK_DGRAM
 from   getpass import getpass, getuser
@@ -112,7 +113,7 @@ class UIO(object):
                 self._print('{}INFO{}:  {}'.format(UIO.GetInfoEscapeSeq(), UIO.DISPLAY_RESET_ESCAPE_SEQ, text))
         else:
             self._print('INFO:  {}'.format(text))
-        self._update_syslog(PRIORITY.INFO, text)
+        self._update_syslog(PRIORITY.INFO, "INFO:  "+text)
 
     def debug(self, text):
         """@brief Present a debug level message to the user if debuging is enabled.
@@ -127,7 +128,7 @@ class UIO(object):
                 self.storeToDebugLog('{}DEBUG{}: {}'.format(UIO.GetDebugEscapeSeq(), UIO.DISPLAY_RESET_ESCAPE_SEQ, text))
             else:
                 self.storeToDebugLog('DEBUG: {}'.format(text))
-        self._update_syslog(PRIORITY.DEBUG, text)
+        self._update_syslog(PRIORITY.DEBUG, "DEBUG: "+text)
 
     def warn(self, text):
         """@brief Present a warning level message to the user.
@@ -136,7 +137,7 @@ class UIO(object):
             self._print('{}WARN{}:  {}'.format(UIO.GetWarnEscapeSeq(), UIO.DISPLAY_RESET_ESCAPE_SEQ, text))
         else:
             self._print('WARN:  {}'.format(text))
-        self._update_syslog(PRIORITY.WARNING, text)
+        self._update_syslog(PRIORITY.WARNING, "WARN:  "+text)
 
     def error(self, text):
         """@brief Present an error level message to the user.
@@ -145,7 +146,7 @@ class UIO(object):
             self._print('{}ERROR{}: {}'.format(UIO.GetErrorEscapeSeq(), UIO.DISPLAY_RESET_ESCAPE_SEQ, text))
         else:
             self._print('ERROR: {}'.format(text))
-        self._update_syslog(PRIORITY.ERROR, text)
+        self._update_syslog(PRIORITY.ERROR, "ERROR: "+text)
 
     def _print(self, text):
         """@brief Print text to stdout"""
@@ -201,20 +202,57 @@ class UIO(object):
             elif allowQuit and response.lower() == 'q':
                 sys.exit(0)
 
-    def getIntInput(self, prompt, allowQuit=True, radix=10):
+    def _getNumericInput(self, floatValue, prompt, allowQuit=True, radix=10, minValue=None, maxValue=None):
       """@brief Get a decimal int number from the user.
+         @param floatValue If True a float value is returned. If False an int value is returned.
          @param allowQuit If True and the user enters q then the program will exit.
-         @param radix The radix of the number entered (default=10).
+         @param radix The radix of the number entered (default=10). Only used if inputting an int value.
+         @param minValue The minimum acceptable value. If left at None then any value is accepted.
+         @param maxValue The maximum acceptable value. If left at None then any value is accepted.
          @return True or False"""
       while True:
         response = self.getInput(prompt=prompt)
         try:
-          return int(response, radix)
-        except ValueError:
-          self.warn("%s is not a valid integer value." % (response))
+            if floatValue:
+                value = float(response)
+            else:
+                value = int(response, radix)
+                
+            if minValue is not None and value < minValue:
+                self.warn(f"The minimum acceptable value is {minValue}")
+                
+            if maxValue is not None and value > maxValue:
+                self.warn(f"The mximum acceptable value is {maxValue}")
+                
+            return value
 
+        except ValueError:
+            if floatValue:
+                self.warn("%s is not a valid float value." % (response))
+                
+            else:
+                self.warn("%s is not a valid integer value." % (response))
+                
         if allowQuit and response.lower() == 'q':
-          return None
+            return None
+
+    def getIntInput(self, prompt, allowQuit=True, radix=10, minValue=None, maxValue=None):
+      """@brief Get a decimal int number from the user.
+         @param allowQuit If True and the user enters q then the program will exit.
+         @param radix The radix of the number entered (default=10).
+         @param minValue The minimum acceptable value. If left at None then any value is accepted.
+         @param maxValue The maximum acceptable value. If left at None then any value is accepted.
+         @return True or False"""
+      return self._getNumericInput(False, prompt, allowQuit=allowQuit, radix=radix, minValue=minValue, maxValue=maxValue)
+
+    def getFloatInput(self, prompt, allowQuit=True, radix=10, minValue=None, maxValue=None):
+      """@brief Get a float number from the user.
+         @param allowQuit If True and the user enters q then the program will exit.
+         @param radix The radix of the number entered (default=10).
+         @param minValue The minimum acceptable value. If left at None then any value is accepted.
+         @param maxValue The maximum acceptable value. If left at None then any value is accepted.
+         @return True or False"""
+      return self._getNumericInput(True, prompt, allowQuit=allowQuit, radix=radix, minValue=minValue, maxValue=maxValue)
 
     def errorException(self):
         """@brief Show an exception traceback if debugging is enabled"""
@@ -273,7 +311,9 @@ class UIO(object):
             # If the log file is about to be created then we will create a symlink
             # to the file.
             if not os.path.isfile(logFile):
-                createSymLink = True
+                # We can't create symlinks on a windows platform
+                if platform.system() != "Windows":
+                    createSymLink = True
 
             with open(logFile, 'a') as fd:
                 if addLF:
@@ -342,7 +382,10 @@ class UIO(object):
            @param syslogProgramName The name of the program that is being logged. If defined this appears after the username in the syslog output."""
         self._sysLogEnabled = enabled
         self._sysLogHost = host
-        self._syslogProgramName = programName
+        if programName:
+            self._syslogProgramName = programName
+        else:
+            self._syslogProgramName = sys.argv[0]
 
     def _update_syslog(self, pri, msg):
         """Send a message to syslog is syslog is enabled
@@ -358,7 +401,7 @@ class UIO(object):
         """
         if self._sysLogEnabled:
             aMsg=msg
-            #Ensure we have no zero characters in the message.
+            #Ensure we have no 0x00 characters in the message.
             # syslog will throw an error if it finds any
             if "\x00" in aMsg:
                 aMsg=aMsg.replace("\x00", "")
@@ -381,7 +424,56 @@ class UIO(object):
             #Ignore if se can't resolve address. We don't really syslog want errors to stop the user interface
             except:
                 pass
+        
+    def showTable(self, table, rowSeparatorChar = "-", colSeparatorChar = "|"):
+        """@brief Show the contents of a table to the user.
+           @param table This must be a list. Each list element must be a table row (list).
+                        Each element in each row must be a string.
+           @param rowSeparatorChar The character used for horizontal lines to separate table rows.
+           @param colSeparatorChar The character used to separate table columns."""
+        columnWidths = []
+        # Check we have a table to display
+        if len(table) == 0:
+            raise Exception("No table rows to display")
+        
+        # Check all rows have the same number of columns in the table
+        colCount = len(table[0])
+        for row in table:
+            if len(row) != colCount:
+                raise Exception(f"{str(row)} column count different from first row ({colCount})")
+        
+        for row in table:
+            for col in row:
+                if not isinstance(col, str):
+                    raise Exception(f"Table column is not a string: {col} in {row}")
+                
+        # Get the max width for each column
+        for col in range(0,colCount):
+            maxWidth=0
+            for row in table:
+                if len(row[col]) > maxWidth:
+                    maxWidth = len(row[col])
+            columnWidths.append(maxWidth)
 
+        tableWidth = 1
+        for columnWidth in columnWidths:
+            tableWidth += columnWidth + 3 # Space each side of the column + a column divider character
+                    
+        # Add the top line of the table
+        self.info(rowSeparatorChar*tableWidth)
+               
+        # The starting row index
+        for rowIndex in range(0, len(table)):
+            rowText = colSeparatorChar
+            colIndex = 0
+            for col in table[rowIndex]:
+                colWidth = columnWidths[colIndex]
+                rowText = rowText + " " + f"{col:>{colWidth}s}" + " " + colSeparatorChar
+                colIndex += 1
+            self.info(rowText)
+            # Add the row separator line
+            self.info(rowSeparatorChar*tableWidth)
+            
 class ConsoleMenu(object):
     """@brief Responsible for presenting a list of options to the user on a
               console/terminal interface and allowing the user to select
@@ -416,6 +508,10 @@ class ConsoleMenu(object):
                     selectedMethod()
                 else:
                     selectedMethod(*args)
+          
+            
+        
+                
 
 # -------------------------------------------------------------------
 # @brief An implementation to allow syslog messages to be generated.
